@@ -24,17 +24,18 @@
 import dbus
 import os as _os
 
+from busutils import *
+
 from factory import AccessibleFactory
 from appevent import _ApplicationEventRegister, _NullApplicationEventRegister
 from deviceevent import _DeviceEventRegister, _NullDeviceEventRegister
-from desktop import *
-from cache import *
+from cache import AccessibleCache
 
 from deviceevent import KEY_PRESSED_EVENT as _KEY_PRESSED_EVENT
 from deviceevent import KEY_RELEASED_EVENT as _KEY_RELEASED_EVENT
 
 from interfaces import ATSPI_REGISTRY_NAME as _ATSPI_REGISTRY_NAME
-from interfaces import ATSPI_DESKTOP_PATH as _ATSPI_DESKTOP_PATH
+from interfaces import ATSPI_ROOT_PATH as _ATSPI_ROOT_PATH
 from interfaces import ATSPI_DESKTOP as _ATSPI_DESKTOP
 
 __all__ = ["Registry",
@@ -115,41 +116,33 @@ class Registry(object):
                 @param app_name: D-Bus name of the application to connect to when not using the registry daemon.
                 """
 
-		factory = AccessibleFactory()
-
-                # Set up the device event controllers
-                if app_name:
-                        devreg = _NullDeviceEventRegister()
-                        appreg = _NullApplicationEventRegister()
-                else:
-                        devreg = _DeviceEventRegister()
-                        appreg = _ApplicationEventRegister()
-
-
-                if app_name:
-                        desktop = TestDesktop (app_name, factory)
-                else:
-                        desktop = Desktop (factory)
 
                 # Set up the cache
 		cache = None
-
                 if main_loop_type == MAIN_LOOP_GLIB:
-                        if app_name:
-                                cache = AccessibleCache(app_name)
-                        else:
-                                cache = ApplicationCache(appreg)
+                                cache = AccessibleCache (app_name)
 
-                factory.set_cache (cache)
-		factory.set_desktop (desktop)
+		factory = AccessibleFactory(cache)
+
+                # Set up the device event controllers
+                _connection = SyncAccessibilityBus ()
+                _bus_object = _connection.get_object("org.freedesktop.DBus", "/org/freedesktop/DBus")
+
+                if app_name:
+                        self.device_event_register = _NullDeviceEventRegister()
+                        self.app_event_register    = _NullApplicationEventRegister()
+
+			name = _bus_object.GetNameOwner (app_name)
+                        self.desktop = factory (name, _ATSPI_ROOT_PATH, _ATSPI_DESKTOP)
+                else:
+                        self.device_event_register = _DeviceEventRegister()
+                        self.app_event_register    = _ApplicationEventRegister(factory)
+
+			name = _bus_object.GetNameOwner (_ATSPI_REGISTRY_NAME)
+                        self.desktop = factory (name, _ATSPI_ROOT_PATH, _ATSPI_DESKTOP)
 
-                # Create the registry object
                 self.has_implementations = True
 
-                self.device_event_register = devreg
-                self.app_event_register = appreg
-                self.desktop = desktop
-
         def _set_default_registry (self):
                 self._set_registry (MAIN_LOOP_GLIB)
 
